/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator.keycloak.user;

import org.springframework.stereotype.Component;

import de.ozgcloud.operator.Config;
import de.ozgcloud.operator.keycloak.OzgCloudCustomResourceStatus;
import io.javaoperatorsdk.operator.api.reconciler.Cleaner;
import io.javaoperatorsdk.operator.api.reconciler.Context;
import io.javaoperatorsdk.operator.api.reconciler.ControllerConfiguration;
import io.javaoperatorsdk.operator.api.reconciler.DeleteControl;
import io.javaoperatorsdk.operator.api.reconciler.Reconciler;
import io.javaoperatorsdk.operator.api.reconciler.UpdateControl;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Log4j2
@RequiredArgsConstructor
@ControllerConfiguration
@Component
public class KeycloakUserReconciler implements Reconciler<OzgCloudKeycloakUser>, Cleaner<OzgCloudKeycloakUser> {

	private final KeycloakUserService service;
	private final KeycloakUserPreconditionService preconditionService;

	@Override
	public UpdateControl<OzgCloudKeycloakUser> reconcile(OzgCloudKeycloakUser resource, Context<OzgCloudKeycloakUser> context) {
		var userName = resource.getMetadata().getName();
		var namespace = resource.getMetadata().getNamespace();

		LOG.info("{} reconciling...", userName);

		try {
			var preconditionError = preconditionService.getPreconditionErrors(resource);
			if (preconditionError.isPresent()) {
				var errorMessage = preconditionError.get();

				LOG.warn("{} could not reconcile in namespace {}: {}.", userName, namespace, errorMessage);

				return UserUpdateControlBuilder.fromResource(resource)
						.withStatus(OzgCloudCustomResourceStatus.IN_PROGRESS)
						.withMessage(errorMessage)
						.withReschedule(Config.RECONCILER_RETRY_SECONDS)
						.build();
			}

			service.createOrUpdateUser(resource.getSpec(), namespace);

			return UserUpdateControlBuilder.fromResource(resource).withStatus(OzgCloudCustomResourceStatus.OK).build();

		} catch (Exception e) {
			LOG.warn(userName + " could not reconcile in namespace " + namespace, e);

			return UserUpdateControlBuilder.fromResource(resource)
					.withStatus(OzgCloudCustomResourceStatus.ERROR)
					.withMessage(e.getMessage())
					.withReschedule(Config.RECONCILER_RETRY_SECONDS_ON_ERROR)
					.build();
		}
	}

	@Override
	public DeleteControl cleanup(OzgCloudKeycloakUser user, Context<OzgCloudKeycloakUser> context) {
		LOG.info("{} cleanup...", user.getMetadata().getName());
		if (user.getSpec().isKeepAfterDelete()) {
			LOG.info("keep data");
			return DeleteControl.defaultDelete();
		}
		return deleteUser(user);
	}

	DeleteControl deleteUser(OzgCloudKeycloakUser user) {
		var userName = user.getMetadata().getName();
		var namespace = user.getMetadata().getNamespace();
		LOG.info("{} do cleanup...", userName);
		try {
			service.deleteUser(user.getSpec(), namespace);
			return DeleteControl.defaultDelete();
		} catch (Exception e) {
			LOG.warn(userName + " could not delete user in namespace " + namespace, e);
			return DeleteControl.noFinalizerRemoval().rescheduleAfter(Config.RECONCILER_RETRY_SECONDS_ON_ERROR);
		}
	}
}
