/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator.keycloak.realm;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.operator.Config;
import de.ozgcloud.operator.keycloak.OzgCloudCustomResourceStatus;
import io.javaoperatorsdk.operator.api.reconciler.DeleteControl;

class KeycloakRealmReconcilerTest {

	@Spy
	@InjectMocks
	private KeycloakRealmReconciler reconciler;

	@Mock
	private KeycloakRealmService service;

	@DisplayName("Reconcile")
	@Nested
	class TestReconcile {

		@Test
		void shouldCallServiceAddRealm() {
			OzgCloudKeycloakRealm realm = OzgCloudKeycloakRealmTestFactory.create();

			reconciler.reconcile(realm, null);

			verify(service).createRealm(realm.getSpec(), OzgCloudKeycloakRealmTestFactory.METADATA_NAMESPACE);
		}

		@Test
		void shouldReturnUpdateStatus() {
			var response = reconciler.reconcile(OzgCloudKeycloakRealmTestFactory.create(), null);

			assertThat(response.getResource()).isNotNull();
		}

		@Test
		void shouldSetStatusOk() {
			var response = reconciler.reconcile(OzgCloudKeycloakRealmTestFactory.create(), null);

			assertThat(response.getResource().getStatus().getStatus()).isEqualTo(OzgCloudCustomResourceStatus.OK);
		}
	}

	@DisplayName("Reconciler Cleanup")
	@Nested
	class TestReconcilerCleanup {

		@DisplayName("with keep_after_delete")
		@Nested
		class TestWithKeepAfterDelete {

			private final OzgCloudKeycloakRealmSpec realmSpec = OzgCloudKeycloakRealmSpecTestFactory.createBuilder().keepAfterDelete(true).build();
			private final OzgCloudKeycloakRealm realm = OzgCloudKeycloakRealmTestFactory.create(realmSpec);

			@Test
			void shouldNotCallService() {
				reconciler.cleanup(realm, null);

				verify(service, never()).deleteRealm(OzgCloudKeycloakRealmTestFactory.METADATA_NAMESPACE);
			}

			@Test
			void shouldReturnDeleteControl() {
				var control = reconciler.cleanup(realm, null);

				assertThat(control).usingRecursiveComparison().isEqualTo(DeleteControl.defaultDelete());
			}
		}

		@DisplayName("test cleanup")
		@Nested
		class TestCleanup {

			private final OzgCloudKeycloakRealm realm = OzgCloudKeycloakRealmTestFactory.create();

			@Test
			void shouldCallDeleteRealm() {
				reconciler.cleanup(realm, null);

				verify(reconciler).deleteRealm(realm);
			}

			@Test
			void shouldReturnValueFromDeleteUser() {
				DeleteControl expected = DeleteControl.defaultDelete();
				when(reconciler.deleteRealm(realm)).thenReturn(expected);

				DeleteControl response = reconciler.cleanup(realm, null);

				assertThat(response).isEqualTo(expected);
			}
		}

		@DisplayName("test delete")
		@Nested
		class TestDelete {

			private final OzgCloudKeycloakRealm realm = OzgCloudKeycloakRealmTestFactory.create();

			@Test
			void shouldCallServiceDelete() {
				reconciler.deleteRealm(realm);

				verify(service).deleteRealm(OzgCloudKeycloakRealmTestFactory.METADATA_NAMESPACE);
			}

			@Test
			void shouldReturnDeleteControl() {
				var control = reconciler.deleteRealm(realm);

				assertThat(control).usingRecursiveComparison().isEqualTo(DeleteControl.defaultDelete());
			}

			@Test
			void shouldRescheduleOnError() {
				doThrow(RuntimeException.class).when(service)
						.deleteRealm(OzgCloudKeycloakRealmTestFactory.METADATA_NAMESPACE);

				var control = reconciler.deleteRealm(realm);

				assertThat(control).usingRecursiveComparison()
						.isEqualTo(DeleteControl.noFinalizerRemoval().rescheduleAfter(Config.RECONCILER_RETRY_SECONDS_ON_ERROR));
			}

			@Test
			void shouldCreateRealmWithSpecWithoutRoles() {
				OzgCloudKeycloakRealm realm = OzgCloudKeycloakRealmTestFactory.create();
				reconciler.reconcile(realm, null);
				var spec = realm.getSpec();
				spec.setRealmRoles(null);
				verify(service).createRealm(spec, OzgCloudKeycloakRealmTestFactory.METADATA_NAMESPACE);
			}
		}
	}
}
